# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: __init__.py,v 1.2 2004/09/16 13:46:09 dreamcatcher Exp $
"""
from _cal import makeRange, DAY, WEEKDAY, WEEK, MONTH, YEAR, HOUR, SECOND

_views = {}

def registerView(component, name=None):
    if name is None:
        name = component.name
    _views[name] = component

def getView(name):
    return _views[name]

def queryView(name, default=None):
    return _views.get(name, default)

from _base import ModelView
