# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: __init__.py,v 1.3 2004/09/17 13:58:25 dreamcatcher Exp $
"""

# Kickstart Extensions.
import Products.Calendaring.Extensions.Install

from Products.Archetypes.public import *
from Products.Calendaring.config import *
from Products.CMFCore import utils as cmf_utils
from Products.CMFCore.DirectoryView import registerDirectory
from Products.CMFPlone import PloneUtilities as plone_utils

registerDirectory('skins', GLOBALS)

def initialize(context):

    from Products.Calendaring.tools import calendar
    from Products.Calendaring.model import year, month, day, hour, week

    tools = (calendar.CalendarTool,)

    plone_utils.ToolInit('Calendaring Tools', tools=tools,
                         product_name='Calendaring', icon='tool.gif',
                         ).initialize(context)
    del calendar

    from Products.Calendaring.content import calendar

    # register archetypes content with the machinery
    content_types, constructors, ftis = process_types(listTypes(PROJECTNAME),
                                                      PROJECTNAME)
    cmf_utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types = content_types,
        permission = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti = ftis).initialize(context)
