# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: day.py,v 1.2 2004/09/16 13:46:09 dreamcatcher Exp $
"""

from dateutil.relativedelta import relativedelta
from Products.Calendaring.model import ModelView, registerView

class OneDayView(ModelView):
    """ A view for a single day """

    unit = 'day'
    _sub = 'hour_view'
    name = 'day_view'
    _args = 3
    _leap = relativedelta(0)

registerView(OneDayView)
