## Controller Python Script "calendar_export"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Export Calendar
##
# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: calendar_export.py,v 1.1.1.1 2004/06/19 14:53:36 dreamcatcher Exp $
"""

REQUEST = context.REQUEST
RESPONSE = REQUEST.RESPONSE
name = context.getId()
if not name.endswith('.ics'):
    name = '%s.ics' % name
RESPONSE.setHeader('content-disposition',
                   'attachment; filename=%s' % name)

return context.asCalendar(REQUEST=context.REQUEST)
